# bglocng17

Background geolocation tracker for internal purposes.

## Install

```bash
npm install bglocng17
npx cap sync
```

## API

<docgen-index>

* [`echo(...)`](#echo)
* [`checkPermissions()`](#checkpermissions)
* [`requestPermissions()`](#requestpermissions)
* [`start(...)`](#start)
* [`stop()`](#stop)
* [`getLocations(...)`](#getlocations)
* [`getStoredLocationsCount()`](#getstoredlocationscount)
* [`getAll()`](#getall)
* [`confirmReception(...)`](#confirmreception)
* [`upload(...)`](#upload)
* [`cleanup(...)`](#cleanup)
* [`addListener(string, ...)`](#addlistenerstring)
* [`on(...)`](#on)
* [Interfaces](#interfaces)
* [Type Aliases](#type-aliases)

</docgen-index>

<docgen-api>
<!--Update the source file JSDoc comments and rerun docgen to update the docs below-->

### echo(...)

```typescript
echo(options: { value: string; }) => Promise<{ value: string; }>
```

| Param         | Type                            |
| ------------- | ------------------------------- |
| **`options`** | <code>{ value: string; }</code> |

**Returns:** <code>Promise&lt;{ value: string; }&gt;</code>

--------------------


### checkPermissions()

```typescript
checkPermissions() => Promise<PermissionStatus>
```

**Returns:** <code>Promise&lt;<a href="#permissionstatus">PermissionStatus</a>&gt;</code>

--------------------


### requestPermissions()

```typescript
requestPermissions() => Promise<PermissionStatus>
```

**Returns:** <code>Promise&lt;<a href="#permissionstatus">PermissionStatus</a>&gt;</code>

--------------------


### start(...)

```typescript
start(config: InfaBgLocConfig) => Promise<void>
```

| Param        | Type                                                        |
| ------------ | ----------------------------------------------------------- |
| **`config`** | <code><a href="#infabglocconfig">InfaBgLocConfig</a></code> |

--------------------


### stop()

```typescript
stop() => Promise<void>
```

--------------------


### getLocations(...)

```typescript
getLocations(options: InfaBgLocFetchOptions) => Promise<InfaBgLocationArray>
```

| Param         | Type                                                                    |
| ------------- | ----------------------------------------------------------------------- |
| **`options`** | <code><a href="#infabglocfetchoptions">InfaBgLocFetchOptions</a></code> |

**Returns:** <code>Promise&lt;<a href="#infabglocationarray">InfaBgLocationArray</a>&gt;</code>

--------------------


### getStoredLocationsCount()

```typescript
getStoredLocationsCount() => Promise<InfaBgLocationCount>
```

**Returns:** <code>Promise&lt;<a href="#infabglocationcount">InfaBgLocationCount</a>&gt;</code>

--------------------


### getAll()

```typescript
getAll() => Promise<InfaBgLocationArray>
```

**Returns:** <code>Promise&lt;<a href="#infabglocationarray">InfaBgLocationArray</a>&gt;</code>

--------------------


### confirmReception(...)

```typescript
confirmReception(options: InfaBgLocConfirmReceptionOptions) => Promise<void>
```

| Param         | Type                                                                                          |
| ------------- | --------------------------------------------------------------------------------------------- |
| **`options`** | <code><a href="#infabglocconfirmreceptionoptions">InfaBgLocConfirmReceptionOptions</a></code> |

--------------------


### upload(...)

```typescript
upload(config: InfaBgLocSessionConfig) => Promise<void>
```

| Param        | Type                                                                      |
| ------------ | ------------------------------------------------------------------------- |
| **`config`** | <code><a href="#infabglocsessionconfig">InfaBgLocSessionConfig</a></code> |

--------------------


### cleanup(...)

```typescript
cleanup(options?: InfaBgLocCleanupSessionOptions | InfaBgLocCleanupByIdsOptions | undefined) => Promise<void>
```

| Param         | Type                                                                                                                                                                  |
| ------------- | --------------------------------------------------------------------------------------------------------------------------------------------------------------------- |
| **`options`** | <code><a href="#infabgloccleanupsessionoptions">InfaBgLocCleanupSessionOptions</a> \| <a href="#infabgloccleanupbyidsoptions">InfaBgLocCleanupByIdsOptions</a></code> |

--------------------


### addListener(string, ...)

```typescript
addListener(eventName: string, listenerFunc: ListenerCallback) => Promise<PluginListenerHandle> & PluginListenerHandle
```

| Param              | Type                                                          |
| ------------------ | ------------------------------------------------------------- |
| **`eventName`**    | <code>string</code>                                           |
| **`listenerFunc`** | <code><a href="#listenercallback">ListenerCallback</a></code> |

**Returns:** <code>Promise&lt;<a href="#pluginlistenerhandle">PluginListenerHandle</a>&gt; & <a href="#pluginlistenerhandle">PluginListenerHandle</a></code>

--------------------


### on(...)

```typescript
on(eventName: string) => Observable<any>
```

| Param           | Type                |
| --------------- | ------------------- |
| **`eventName`** | <code>string</code> |

**Returns:** <code>Observable&lt;any&gt;</code>

--------------------


### Interfaces


#### PermissionStatus

| Prop           | Type                                                        |
| -------------- | ----------------------------------------------------------- |
| **`location`** | <code><a href="#permissionstate">PermissionState</a></code> |


#### InfaBgLocConfig

Used to configure the background service


#### InfaBgLocationArray

| Prop        | Type                          |
| ----------- | ----------------------------- |
| **`items`** | <code>InfaBgLocation[]</code> |


#### InfaBgLocation

| Prop                       | Type                |
| -------------------------- | ------------------- |
| **`id`**                   | <code>number</code> |
| **`sessionToken`**         | <code>string</code> |
| **`time`**                 | <code>number</code> |
| **`latitude`**             | <code>number</code> |
| **`longitude`**            | <code>number</code> |
| **`altitude`**             | <code>number</code> |
| **`speed`**                | <code>number</code> |
| **`bearing`**              | <code>number</code> |
| **`angularSpeed`**         | <code>number</code> |
| **`accuracy`**             | <code>number</code> |
| **`verticalAccuracy`**     | <code>number</code> |
| **`speedAccuracy`**        | <code>number</code> |
| **`bearingAccuracy`**      | <code>number</code> |
| **`angularSpeedAccuracy`** | <code>number</code> |
| **`nanos`**                | <code>number</code> |
| **`provider`**             | <code>string</code> |
| **`latitudeRaw`**          | <code>number</code> |
| **`longitudeRaw`**         | <code>number</code> |


#### InfaBgLocFetchOptions

| Prop                         | Type                 | Description                                                                               | Default            |
| ---------------------------- | -------------------- | ----------------------------------------------------------------------------------------- | ------------------ |
| **`sessionToken`**           | <code>string</code>  | Set the session to retrieve the locations for. (required)                                 |                    |
| **`limit`**                  | <code>number</code>  | Limits the number of returned locations and retrieves the oldest entries.                 | <code>null</code>  |
| **`ignoreTransmissionFlag`** | <code>boolean</code> | Get the entire dataset for this session, regardless of the current app-transmission flag. | <code>false</code> |


#### InfaBgLocationCount

| Prop        | Type                |
| ----------- | ------------------- |
| **`count`** | <code>number</code> |


#### InfaBgLocConfirmReceptionOptions

| Prop               | Type                  | Description                                                                                                                   |
| ------------------ | --------------------- | ----------------------------------------------------------------------------------------------------------------------------- |
| **`sessionToken`** | <code>string</code>   | Set the session to confirm the reception                                                                                      |
| **`ids`**          | <code>number[]</code> | Set the array of ids handled within the application to mark all locations within the native plugin as handled up to this one. |


#### InfaBgLocSessionConfig

| Prop                            | Type                 | Description                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        | Default                  |
| ------------------------------- | -------------------- | ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------ | ------------------------ |
| **`sessionToken`**              | <code>string</code>  |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    |                          |
| **`httpUrl`**                   | <code>string</code>  | If set, the service will post the recorded locations to the given server url end-point as soon as there is a valid internet connection available. The data will be sent as POST-Method and (actually) without any authorization header. Therefore the end-point should allow anonymous access.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | <code>null</code>        |
| **`httpPostTemplate`**          | <code>string</code>  | Defines the JSON-based template used for sending a location to the server. Please use the following wildcards as desired: - %id% (the auto-incremental identifier for every recorded location) - %sessionToken% (the identifier used for the particular recording session) - %apiSendStatus% (retrieves 0 or 1, whether the location has been flagged to already be sent to the server) - %appSendStatus% (retrieves 0 or 1, whether the location has been flagged to already be collected by the app itself) - %time% (the location's UTC time fix, in milliseconds since epoch (January 1, 1970).) - %latitude% (the latitude, in degrees) - %longitude% (the longitude, in degrees) - %altitude% (the altitude if available, in meters above the WGS 84 reference ellipsoid.) - %speed% (the speed if it is available, in meters/second over ground.) - %bearing% (the bearing, in degrees [0..360]) - %horizontalAccuracy% (the estimated horizontal accuracy of this location, radial, in meters.) - %verticalAccuracy% (the estimated vertical accuracy of this location, in meters.) - %speedAccuracy% (the estimated speed accuracy of this location, in meters per second.) - %bearingAccuracy% (the estimated bearing accuracy of this location, in degrees.) - %nanos% (the time of this fix, in elapsed real-time since system boot.) - %provider% (the name of the provider that generated this fix.) |                          |
| **`httpAuthentication`**        | <code>string</code>  |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    |                          |
| **`httpPostLimit`**             | <code>number</code>  | Defines the number of locations maximal sent to the server at once.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                | <code>MAX_INTEGER</code> |
| **`httpPostFrequency`**         | <code>number</code>  | Defines the server upload frequency in milliseconds.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               | <code>10000</code>       |
| **`idleTimeUntilNotification`** | <code>number</code>  | When the location has not changed during the given amount of milliseconds, the service returns a notification. The default time is 4620000                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         |                          |
| **`waitForClientCollection`**   | <code>boolean</code> | Indicates, whether a location shall wait for the client to be collected, before it can be (regularly) cleared from the storage.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    |                          |
| **`backgroundService`**         | <code>boolean</code> | Allows the service to persist in the background, although the main app itself might have been shut down already.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   | <code>false</code>       |


#### InfaBgLocCleanupSessionOptions

| Prop                | Type                 | Description                                                                                                                                      | Default           |
| ------------------- | -------------------- | ------------------------------------------------------------------------------------------------------------------------------------------------ | ----------------- |
| **`sessionToken`**  | <code>string</code>  | The session to be cleaned up. (required)                                                                                                         |                   |
| **`minSessionAge`** | <code>number</code>  | Defines the minimum age of the session in days, before it can be discarded. A value of 0 allows the service to immediately delete the location.  | <code>0</code>    |
| **`forceCleanup`**  | <code>boolean</code> | All locations have to be posted to a server (through HTTP-POST) by the native upload client, before any location of the session will be deleted. | <code>true</code> |


#### InfaBgLocCleanupByIdsOptions

| Prop      | Type                  | Description               |
| --------- | --------------------- | ------------------------- |
| **`ids`** | <code>number[]</code> | The ids to be cleaned up. |


#### PluginListenerHandle

| Prop         | Type                                      |
| ------------ | ----------------------------------------- |
| **`remove`** | <code>() =&gt; Promise&lt;void&gt;</code> |


### Type Aliases


#### PermissionState

<code>'prompt' | 'prompt-with-rationale' | 'granted' | 'denied'</code>


#### ListenerCallback

<code>(err: any, ...args: any[]): void</code>

</docgen-api>
