package de.infa.plugins.bgloc.service;

import android.Manifest;
import android.annotation.SuppressLint;
import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.location.Location;
import android.os.Build;
import android.os.IBinder;
import android.util.Log;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.core.app.ActivityCompat;
import androidx.core.app.NotificationCompat;

import com.google.android.gms.location.FusedLocationProviderClient;
import com.google.android.gms.location.LocationAvailability;
import com.google.android.gms.location.LocationCallback;
import com.google.android.gms.location.LocationRequest;
import com.google.android.gms.location.LocationResult;
import com.google.android.gms.location.LocationServices;

import java.util.Timer;
import java.util.TimerTask;

import de.infa.plugins.bgloc.R;
import de.infa.plugins.bgloc.service.configuration.SessionConfiguration;
import de.infa.plugins.bgloc.service.exceptions.ServiceConfigurationException;
import de.infa.plugins.bgloc.service.storage.SQLiteRepository;

public class LocationService extends Service {
  public static final String INTENT_START_SERVICE = "INFA_BGLOC_LOCATION_SERVICE_START";
  public static final String INTENT_STOP_SERVICE = "INFA_BGLOC_LOCATION_SERVICE_STOP";
  public static final String INTENT_LOCATION_UPDATE = "INFA_BGLOC_INTENT_LOCATION_UPDATE";
  public static final String INTENT_LOCATION_IDLE_TIME_EXCEEDED = "INFA_BGLOC_INTENT_LOCATION_IDLE_TIME_EXCEEDED";

  public static final String INTENT_START_EXTRA_LOCATION_REQUEST = "INFA_BGLOC_INTENT_START_EXTRA_LOCATION_REQUEST";
  public static final String INTENT_START_EXTRA_SESSION_CONFIG = "INFA_BGLOC_INTENT_START_EXTRA_SESSION_CONFIG";

  public static final String INTENT_LOCATION_UPDATE_EXTRA_ID = "INFA_BGLOC_INTENT_LOCATION_UPDATE_EXTRA_ID";
  public static final String INTENT_LOCATION_UPDATE_EXTRA_SESSION = "INFA_BGLOC_INTENT_LOCATION_UPDATE_EXTRA_SESSION";
  public static final String INTENT_LOCATION_UPDATE_EXTRA_LOCATION = "INFA_BGLOC_INTENT_LOCATION_UPDATE_EXTRA_LOCATION";

  public static final String INTENT_LOCATION_IDLE_TIME_EXCEEDED_EXTRA_SESSION = "INFA_BGLOC_INTENT_LOCATION_IDLE_TIME_EXCEEDED_EXTRA_SESSION";
  public static final String INTENT_LOCATION_IDLE_TIME_EXCEEDED_EXTRA_TIME = "INFA_BGLOC_INTENT_LOCATION_IDLE_TIME_EXCEEDED_EXTRA_TIME";

  public LocationService() {
  }

  @Override
  public IBinder onBind(Intent intent) {
    throw new UnsupportedOperationException(getString(R.string.location_service_bind_exception_msg));
  }

  private LocationRequest mLocationRequest;
  //  private float mSimulatedSmallestDisplacement;
  private SessionConfiguration mSessionConfiguration;
  private FusedLocationProviderClient mProvider;
  @Override
  public int onStartCommand(Intent intent, int flags, int startId) throws ServiceConfigurationException {
    if (intent != null) {
      String action = intent.getAction();
      if (action != null) {
        switch (action) {
          case INTENT_START_SERVICE:
            mLocationRequest = intent.getParcelableExtra(INTENT_START_EXTRA_LOCATION_REQUEST);
            if (mLocationRequest == null) {
              Log.e(getString(R.string.package_name_short), getString(R.string.location_service_start_missing_request_config_exception_msg));
              throw new ServiceConfigurationException(getString(R.string.location_service_start_missing_request_config_exception_msg));
            }
            mSessionConfiguration = intent.getParcelableExtra(INTENT_START_EXTRA_SESSION_CONFIG);
            if (mSessionConfiguration == null) {
              Log.e(getString(R.string.package_name_short), getString(R.string.location_service_start_missing_session_config_exception_msg));
              throw new ServiceConfigurationException(getString(R.string.location_service_start_missing_session_config_exception_msg));
            } else if (mSessionConfiguration.getSessionToken() == null) {
              Log.e(getString(R.string.package_name_short), getString(R.string.location_service_start_missing_session_token_exception_msg));
              throw new ServiceConfigurationException(getString(R.string.location_service_start_missing_session_token_exception_msg));
            }
            if (mSessionConfiguration.getBackgroundService()) {
              doStartForeground();
            }
            doStartTracker();
            doStartTimer();
            break;
          case INTENT_STOP_SERVICE:
            doStopTimer();
            doStopTracker();
            if (mSessionConfiguration.getBackgroundService()) {
              doStopForeground();
            }
            break;
        }
      }
    }
    return START_STICKY;
    //return super.onStartCommand(intent, flags, startId);
  }

  @SuppressLint("VisibleForTests")
  private void doStartTracker() throws UnsupportedOperationException {
    if (mProvider != null) {
      Log.w(getString(R.string.package_name_short), getString(R.string.location_service_start_already_running_warning_log));
      return;
    }

    if (ActivityCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
      throw new UnsupportedOperationException(String.format(getString(R.string.location_service_missing_permission_exception_msg), "ACCESS_FINE_LOCATION"));
    }
    if (ActivityCompat.checkSelfPermission(this, Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
      throw new UnsupportedOperationException(String.format(getString(R.string.location_service_missing_permission_exception_msg), "ACCESS_COARSE_LOCATION"));
    }
    if (ActivityCompat.checkSelfPermission(this, Manifest.permission.ACCESS_BACKGROUND_LOCATION) != PackageManager.PERMISSION_GRANTED) {
      throw new UnsupportedOperationException(String.format(getString(R.string.location_service_missing_permission_exception_msg), "ACCESS_BACKGROUND_LOCATION"));
    }

    mProvider = LocationServices.getFusedLocationProviderClient(LocationService.this);
    mProvider.requestLocationUpdates(mLocationRequest, callbackHandler, null);
    if (mSessionConfiguration.getBackgroundService()) {
      Toast.makeText(getApplicationContext(), R.string.bg_location_service_started_toast_msg, Toast.LENGTH_LONG).show();
    } else {
      Toast.makeText(getApplicationContext(), R.string.location_service_started_toast_msg, Toast.LENGTH_LONG).show();
    }
  }

  private void doStartForeground()  {
    Notification mNotification = getNotification();
    startForeground(1, mNotification);
  }

  private Timer mTimer;

  private void doStartTimer() {
    if (mSessionConfiguration.getIdleTimeUntilNotification() < 1) {
      return;
    }
    mTimer = new Timer();
    mTimer.schedule(new TimerTask() {
      @Override
      public void run() {
        Intent intent = new Intent();
        intent.setAction(INTENT_LOCATION_IDLE_TIME_EXCEEDED);
        sendBroadcast(intent);
      }
    }, mSessionConfiguration.getIdleTimeUntilNotification());
  }

  private void doResetTimer() {
    doStopTimer();
    doStartTimer();
  }

  private void doStopTimer() {
    if (mTimer != null) {
      mTimer.cancel();
      mTimer = null;
    }
  }

  private void doStopTracker() {
    if (mProvider != null) {
      mProvider.removeLocationUpdates(callbackHandler);
      mProvider = null;
      if (mSessionConfiguration.getBackgroundService()) {
        Toast.makeText(getApplicationContext(), R.string.bg_location_service_stopped_toast_msg, Toast.LENGTH_LONG).show();
      }
    }
  }

  private void doStopForeground() {
    stopForeground(true);
    stopSelf();
  }

  private final LocationCallback callbackHandler = new LocationCallback() {
    @Override
    public void onLocationResult(LocationResult locationResult) {
      Location lastLocation = locationResult.getLastLocation();
      handleLocationForeground(lastLocation);
    }

    @Override
    public void onLocationAvailability(@NonNull LocationAvailability locationAvailability) {
      // do nothing
    }
  };

  private boolean isRelevantLocation() {
    return true;
  }

  private void handleLocationForeground(Location location) {
    String sessionToken = mSessionConfiguration.getSessionToken();
    if (!isRelevantLocation()) {
//      Log.d(getString(R.string.package_name_short), String.format("Session: %s   %.4f / %.4f (skipped due to missing movement)", sessionToken, location.getLongitude(), location.getLatitude()));
      return;
    }

    SQLiteRepository rec = new SQLiteRepository();
    Boolean appSendFlag = mSessionConfiguration.getWaitForClientCollection() ? false : null;
    Boolean apiSendFlag = mSessionConfiguration.getHttpUrl() != null ? false : null;
    int id = rec.put(this, location, sessionToken, apiSendFlag, appSendFlag);
    if (id > -1) {
      Intent intent = new Intent();
      intent.setAction(INTENT_LOCATION_UPDATE);
      intent.putExtra(INTENT_LOCATION_UPDATE_EXTRA_ID, id);
      intent.putExtra(INTENT_LOCATION_UPDATE_EXTRA_SESSION, sessionToken);
      intent.putExtra(INTENT_LOCATION_UPDATE_EXTRA_LOCATION, location);
      sendBroadcast(intent);
    }
    doResetTimer();
//    Log.d(getString(R.string.package_name_short), String.format(Locale.GERMAN, "Session: %s   %.4f / %.4f", sessionToken, location.getLongitude(), location.getLatitude()));
  }

  private Notification getNotification() {
    Intent intent = new Intent();
    PendingIntent pendingIntent = PendingIntent.getActivity(LocationService.this, 0, intent, PendingIntent.FLAG_IMMUTABLE);

    NotificationCompat.Builder builder;
    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
      NotificationChannel channel = new NotificationChannel(
        String.format("%s_CHANNEL_ID", this.getPackageName().toUpperCase()),
        String.format("%s Service notification channel", getString(R.string.package_name_short)), NotificationManager.IMPORTANCE_HIGH);
      NotificationManager notificationManager = (NotificationManager) this.getSystemService(Context.NOTIFICATION_SERVICE);
      if (notificationManager != null) {
        notificationManager.createNotificationChannel(channel);
      }
      builder = new NotificationCompat.Builder(LocationService.this, channel.getId());
    }
    else {
      builder =  new NotificationCompat.Builder(LocationService.this, String.format("%s_CHANNEL_ID", this.getPackageName().toUpperCase()));
    }

    NotificationCompat.BigTextStyle bigTextStyle = new NotificationCompat.BigTextStyle();
    bigTextStyle.setBigContentTitle(getString(R.string.package_name));
    bigTextStyle.bigText(getString(R.string.location_service_notification_bigtext));
    builder.setStyle(bigTextStyle);

    builder.setWhen(System.currentTimeMillis());
    builder.setSmallIcon(R.drawable.ic_baseline_location_on_24_w80);
    Bitmap largeIconBitmap = BitmapFactory.decodeResource(this.getResources(), R.drawable.ic_baseline_location_on_24_w80);
    builder.setLargeIcon(largeIconBitmap);

    builder.setPriority(Notification.PRIORITY_MAX);

    builder.setFullScreenIntent(pendingIntent, true);
    return builder.build();
  }
}
