package de.infa.plugins.bgloc.service.configuration;

import android.content.Context;

import com.getcapacitor.PluginCall;

import org.jetbrains.annotations.NotNull;

import de.infa.plugins.bgloc.R;
import de.infa.plugins.bgloc.service.exceptions.ServiceConfigurationException;

public class PluginConfigurationReader {
  public ServiceConfiguration getServiceConfiguration(@NotNull final Context context, final PluginCall call) {
    ServiceConfiguration config = new ServiceConfiguration();
    if (call.hasOption("interval")) {
      config.setInterval(call.getInt("interval"));
    } else {
      throw new ServiceConfigurationException(String.format(context.getString(R.string.location_service_start_missing_service_configuration_parameter_exception_msg), "interval"));
    }
    if (call.hasOption("fastestInterval")) {
      config.setFastestInterval(call.getInt("fastestInterval"));
    }
    if (call.hasOption("smallestDisplacement")) {
      config.setSmallestDisplacement(call.getFloat("smallestDisplacement"));
    }
    if (call.hasOption("priority")) {
      config.setPriority(call.getInt("priority"));
    }
    if (call.hasOption("expiration")) {
      config.setExpiration(call.getInt("expiration", Integer.MAX_VALUE));
    }
    return config;
  }

  public SessionConfiguration getSessionConfiguration(@NotNull final Context context, final PluginCall call) {
    SessionConfiguration config = new SessionConfiguration();
    if (call.hasOption("sessionToken")) {
      config.setSessionToken(call.getString("sessionToken", null));
    } else {
      throw new ServiceConfigurationException(String.format(context.getString(R.string.location_service_start_missing_service_configuration_parameter_exception_msg), "sessionToken"));
    }
    config.setHttpUrl(call.getString("httpUrl", null));
    config.setHttpTokenRefreshUrl(call.getString("httpTokenRefreshUrl", null));
    config.setHttpAuthentication(call.getString("httpAuthentication", null));
    config.setHttpPostTemplate(call.getString("httpPostTemplate", config.getDefaultHttpPostTemplate()));
    if (call.hasOption("httpPostLimit")) {
      config.setHttpPostLimit(call.getInt("httpPostLimit", Integer.MAX_VALUE));
    }
    if (call.hasOption("httpPostFrequency")) {
      config.setHttpPostFrequency(call.getInt("httpPostFrequency", 10000));
    }
    if (call.hasOption("waitForClientCollection")) {
      config.setWaitForClientCollection(call.getBoolean("waitForClientCollection", true));
    } else {
      config.setWaitForClientCollection(true);
    }
    if (call.hasOption("idleTimeUntilNotification")) {
      config.setIdleTimeUntilNotification(call.getInt("idleTimeUntilNotification", 43200000)); // 12h
    }
    if (call.hasOption("backgroundService")) {
      config.setBackgroundService(call.getBoolean("backgroundService", false));
    }

    return config;
  }
}
