package de.infa.plugins.bgloc.service.configuration;

import android.os.Build;
import android.os.Parcel;
import android.os.Parcelable;

import org.json.JSONException;
import org.json.JSONObject;

import androidx.annotation.RequiresApi;

public class SessionConfiguration implements Parcelable {
  private String mSessionToken;
  private String mHttpPostTemplate;
  private String mHttpUrl;
  private String mHttpTokenRefreshUrl;
  private String mHttpAuthentication;
  private int mHttpPostLimit;
  private int mHttpPostFrequency;
  private boolean mWaitForClientCollection;
  private int mIdleTimeUntilNotification;
  private boolean mBackgroundService;

  public SessionConfiguration() {
    mHttpPostLimit = Integer.MAX_VALUE;
    mHttpPostFrequency = 10000;
    mIdleTimeUntilNotification = 43200000;
    mBackgroundService = false;
  }

  @RequiresApi(api = Build.VERSION_CODES.Q)
  protected SessionConfiguration(Parcel in) {
    mSessionToken = in.readString();
    mHttpPostTemplate = in.readString();
    mHttpUrl = in.readString();
    mHttpAuthentication = in.readString();
    mHttpPostLimit = in.readInt();
    mHttpPostFrequency = in.readInt();
    mWaitForClientCollection = in.readByte() != 0;
    mIdleTimeUntilNotification = in.readInt();
    mBackgroundService = in.readBoolean();
    mHttpTokenRefreshUrl = in.readString();
  }

  public static final Creator<SessionConfiguration> CREATOR = new Creator<SessionConfiguration>() {
    @RequiresApi(api = Build.VERSION_CODES.Q)
    @Override
    public SessionConfiguration createFromParcel(Parcel in) {
      return new SessionConfiguration(in);
    }

    @Override
    public SessionConfiguration[] newArray(int size) {
      return new SessionConfiguration[size];
    }
  };

  public String getSessionToken() { return mSessionToken; }
  public void setSessionToken(String value) { mSessionToken = value; }
  public String getHttpPostTemplate() { return mHttpPostTemplate; }
  public void setHttpPostTemplate(String value) { mHttpPostTemplate = value; }
  public String getHttpUrl() { return mHttpUrl; }
  public void setHttpUrl(String value) { mHttpUrl = value; }
  public String getHttpTokenRefreshUrl() { return mHttpTokenRefreshUrl; }
  public void setHttpTokenRefreshUrl(String value) { mHttpTokenRefreshUrl = value; }
  public String getHttpAuthentication() { return mHttpAuthentication; }
  public void setHttpAuthentication(String value) { mHttpAuthentication = value; }
  public Integer getHttpPostLimit() { return mHttpPostLimit; }
  public void setHttpPostLimit(Integer value) { mHttpPostLimit = value != null ? value : Integer.MAX_VALUE; }
  public int getHttpPostFrequency() { return mHttpPostFrequency; }
  public void setHttpPostFrequency(int value) { mHttpPostFrequency = value; }
  public boolean getWaitForClientCollection() { return mWaitForClientCollection; }
  public void setWaitForClientCollection(boolean value) { mWaitForClientCollection = value; }
  public Integer getIdleTimeUntilNotification() { return mIdleTimeUntilNotification; }
  public void setIdleTimeUntilNotification(Integer value) { mIdleTimeUntilNotification = value != null ? value : 43200000; }
  public boolean getBackgroundService() { return mBackgroundService; }
  public void setBackgroundService(boolean value) { mBackgroundService = value; }

  public String getDefaultHttpPostTemplate() {
    JSONObject template = new JSONObject();
    try {
      template.put("id", "@id");
      template.put("sessionToken", "@sessionToken");
      template.put("latitude", "@latitude");
      template.put("longitude", "@longitude");
      template.put("altitude", "@altitude");
      template.put("time", "@time");
      return template.toString();
    } catch (JSONException e) {
      e.printStackTrace();
      return null;
    }
  }

  @Override
  public int describeContents() {
    return 0;
  }

  @RequiresApi(api = Build.VERSION_CODES.Q)
  @Override
  public void writeToParcel(Parcel dest, int flags) {
    dest.writeString(mSessionToken);
    dest.writeString(mHttpPostTemplate);
    dest.writeString(mHttpUrl);
    dest.writeString(mHttpAuthentication);
    dest.writeInt(mHttpPostLimit);
    dest.writeInt(mHttpPostFrequency);
    dest.writeBoolean(mWaitForClientCollection);
    dest.writeInt(mIdleTimeUntilNotification);
    dest.writeBoolean(mBackgroundService);
    dest.writeString(mHttpTokenRefreshUrl);
  }
}
