package de.infa.plugins.bgloc.service.storage;

import java.util.ArrayList;

public class SQLiteGetLocationsOptions {
  private String mSessionToken;
  private Integer mLimit;
  private Boolean mIgnoreTransmissionFlag;
  private SQLiteGetLocationsRecipient mRecipient;

  public SQLiteGetLocationsOptions() {
    mLimit = 100;
    mIgnoreTransmissionFlag = false;
    mRecipient = SQLiteGetLocationsRecipient.APP;
  }

  public String getSessionToken() { return mSessionToken; }
  public SQLiteGetLocationsOptions setSessionToken(String value) { mSessionToken = value; return this; }
  public Integer getLimit() { return mLimit; }
  public SQLiteGetLocationsOptions setLimit(Integer value) { mLimit = value; return this; }
  public Boolean getIgnoreTransmissionFlag() { return mIgnoreTransmissionFlag; }
  public SQLiteGetLocationsOptions setIgnoreTransmissionFlag(Boolean value) { mIgnoreTransmissionFlag = value; return this; }
  public SQLiteGetLocationsRecipient getRecipient() { return mRecipient; }
  public SQLiteGetLocationsOptions setRecipient(SQLiteGetLocationsRecipient value) { mRecipient = value; return this; }

  public String getQuerySelection() {
    ArrayList<String> list = new ArrayList<>();
    if (mSessionToken != null) {
      list.add("session_token = ?");
    }
    if (mIgnoreTransmissionFlag == null || !mIgnoreTransmissionFlag) {
      // wenn nicht alle aufgezeichneten Einträge der Session in Frage kommen, dann sollen nur jene
      // zurückgegeben werden, die noch nicht an den Empfänger gesendet wurden.
      switch (mRecipient) {
        case API:
          list.add("api_transmission_flag = 0");
          break;
        case APP:
          list.add("app_transmission_flag = 0");
          break;
      }
    }
    return list.size() > 0 ? String.join(" and ", list) : null;
  }

  public String[] getQuerySelectionArgs() {
    ArrayList<String> list = new ArrayList<>();
    if (mSessionToken != null) {
      list.add(mSessionToken);
    }
    return list.size() > 0 ? list.stream().toArray(String[]::new) : null;
  }

  public String getLimitArgs() {
    return mLimit != null ? mLimit.toString() : null;
  }
}
