package de.infa.plugins.bgloc.service.storage;

import android.content.Context;
import android.database.Cursor;
import android.util.Log;

import org.jetbrains.annotations.NotNull;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.Locale;

import de.infa.plugins.bgloc.R;

public abstract class SQLiteHttpPostTemplateReader extends SQLiteObjectReader<JSONObject> {
  public abstract String getHttpPostTemplate();

  private String mFormattableHttpPostTemplate;
  private String getFormattableHttpPostTemplate() {
    if (mFormattableHttpPostTemplate == null) {
      mFormattableHttpPostTemplate = getHttpPostTemplate()
        .replace("\"@id\"", "%1$d")
        .replace("@sessionToken", "\"%2$s\"")
        .replace("\"@apiTransmissionFlag\"", "%3$d")
        .replace("\"@appTransmissionFlag\"", "%4$d")
        .replace("\"@time\"", "%5$d")
        .replace("\"@latitude\"", "%6$f")
        .replace("\"@longitude\"", "%7$f")
        .replace("\"@altitude\"", "%8$f")
        .replace("\"@speed\"", "%9$f")
        .replace("\"@bearing\"", "%10$f")
        .replace("\"@accuracy\"", "%11$f")
        .replace("\"@verticalAccuracy\"", "%12$f")
        .replace("\"@speedAccuracy\"", "%13$f")
        .replace("\"@bearingAccuracy\"", "%14$f")
        .replace("\"@nanos\"", "%15$d")
        .replace("@provider", "\"%16$s\"");
    }
    return mFormattableHttpPostTemplate;
  }

  public boolean validateHttpPostTemplate(Context context) {
    try {
      String formatted = String.format(getFormattableHttpPostTemplate(),
        1, // cursor.getInt(0),
        "session", // cursor.getString(1),
        2, // cursor.getInt(2),
        3, // cursor.getInt(3),
        4, // cursor.getInt(4),
        5.5f, // cursor.getFloat(5),
        6.6f, // cursor.getFloat(6),
        7.7f, // cursor.getFloat(7),
        8.8f, // cursor.getFloat(8),
        9.9f, // cursor.getFloat(9),
        10.1f, // cursor.getFloat(10),
        11.11f, // cursor.getFloat(11),
        12.12f, // cursor.getFloat(12),
        13.13f, // cursor.getFloat(13),
        123456789012345L, // cursor.getLong(14),
        "provider" // cursor.getString(15)
      );
      JSONObject json = new JSONObject(formatted);
      return true;
    } catch (Exception ex) {
      if (context != null) {
        Log.e(context.getString(R.string.package_name_short), "Post template validation failed.", ex);
      } else {
        ex.printStackTrace();
      }
      return false;
    }
  }

  @Override
  public JSONObject createFrom(@NotNull Cursor cursor) {
    String formatted = String.format(Locale.US, getFormattableHttpPostTemplate(),
      cursor.getInt(0),
      cursor.getString(1),
      cursor.getInt(2),
      cursor.getInt(3),
      cursor.getLong(4),
      cursor.getFloat(5),
      cursor.getFloat(6),
      cursor.getFloat(7),
      cursor.getFloat(8),
      cursor.getFloat(9),
      cursor.getFloat(10),
      cursor.getFloat(11),
      cursor.getFloat(12),
      cursor.getFloat(13),
      cursor.getLong(14),
      cursor.getString(15)
    );
    try {
      JSONObject result = new JSONObject(formatted);
      return result;
    } catch (JSONException e) {
      e.printStackTrace();
      return null;
    }
  }
}
