package de.infa.plugins.bgloc.service.storage;

import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;

public abstract class SQLiteObjectReader<T extends Object> extends SQLiteRepository {
  public abstract T createFrom(@NotNull Cursor cursor);

  public ArrayList<T> getList(@NotNull final Context context, @Nullable SQLiteGetLocationsOptions options) {
    try (SQLiteDatabase db = getDatabase(context)) {
      ArrayList<T> arr = new ArrayList<>();
      if (tableExists(db)) {
        try (Cursor cursor = db.query(TABLE_NAME,
                new String[]{
                        "id", // 0
                        "session_token",
                        "api_transmission_flag",
                        "app_transmission_flag",
                        "time",
                        "latitude", // 5
                        "longitude",
                        "altitude",
                        "speed",
                        "bearing",
                        "h_acc", // 10
                        "v_acc",
                        "speed_acc",
                        "bearing_acc",
                        "nanos",
                        "provider" // 15
                },
                options != null ? options.getQuerySelection() : null,
                options != null ? options.getQuerySelectionArgs() : null,
                null, null, "nanos",
                options != null ? options.getLimitArgs() : null
        )) {
          cursor.moveToFirst();
          while (!cursor.isAfterLast()) {
            T obj = createFrom(cursor);
            arr.add(obj);
            cursor.moveToNext();
          }
        }
      }
      return arr;
    }
  }

  public boolean hasEntries(@NotNull final Context context, String sessionToken, SQLiteGetLocationsRecipient recipient) {
    try (SQLiteDatabase db = getDatabase(context)) {
      if (tableExists(db)) {
        SQLiteGetLocationsOptions options = new SQLiteGetLocationsOptions();
        options.setSessionToken(sessionToken);
        options.setRecipient(recipient);
        try (Cursor cursor = db.query(TABLE_NAME,
                new String[]{ "id" },
                options.getQuerySelection(),
                options.getQuerySelectionArgs(),
                null, null, null, "1"
        )) {
          cursor.moveToFirst();
          return !cursor.isAfterLast();
        }
      }
      return false;
    }
  }
}
