import { BehaviorSubject, Observable, Subject } from 'rxjs';
import { InfaBgLocation, InfaBgLocationArray, InfaBgLocCleanupByIdsOptions, InfaBgLocCleanupSessionOptions, InfaBgLocConfig, InfaBgLocConfirmReceptionOptions, InfaBgLocEvent, InfaBgLocFetchOptions, InfaBgLocIdleTimeExceededResponse, InfaBgLocSessionConfig } from './models/interfaces';
export declare class InfaBgLocService {
    private timer;
    private waitingForResponse;
    private stopFlag;
    private handler?;
    active: BehaviorSubject<boolean>;
    location: Subject<InfaBgLocation>;
    idleTimeout: Subject<InfaBgLocIdleTimeExceededResponse>;
    constructor();
    /** Start the native background-location service by using the parameterized configuration.
     * Define a handler, if you want to process the locations within your app safely. The
     * returned numbers will then be confirmed to the native plugin.
     */
    start(config: InfaBgLocConfig, handler?: (locations: InfaBgLocationArray) => Promise<number[]>, limit?: number): Promise<void>;
    /** Stop the native background-location service. */
    stop(): void;
    private doStop;
    /** Clean up the plugin repository. Only succeeds, if all locations
     * have been transmitted already or if you force the cleanup through the
     * parameter for a specific session.
     */
    cleanup(options?: InfaBgLocCleanupSessionOptions | InfaBgLocCleanupByIdsOptions): Promise<void>;
    /** Start the http service without starting the location service itself.
     * When every location has been uploaded to the server, the service
     * shuts down.
     */
    upload(config: InfaBgLocSessionConfig): Promise<void>;
    /** If options are set, returns the unconfirmed locations of the given session.
     * Otherwise, returns all locations still stored in the repository.
     */
    get(options?: InfaBgLocFetchOptions): Promise<InfaBgLocationArray>;
    getStoredLocationsCount(): Promise<number>;
    /** Confirm the locations to be received by the app. Otherwise, they
     * will be returned again with the next get() request.
     */
    confirm(options: InfaBgLocConfirmReceptionOptions): Promise<void>;
    /** Provides an event based update mechanism. */
    on(event: InfaBgLocEvent): Observable<any>;
    /** Requests the required permissions, just in case, that they are not granted already. */
    requestPermissions(): Promise<void>;
    /** Answers true, when all required permissions are already granted. */
    checkPermissions(): Promise<boolean>;
}
