import { WebPlugin, } from '@capacitor/core';
//@ts-ignore
import { Observable, Subject } from 'rxjs';
import { InfaBgLocEvent, } from './models/interfaces';
export class BgLocWeb extends WebPlugin {
    checkPermissions() {
        return Promise.resolve({ location: 'granted' });
    }
    requestPermissions() {
        return Promise.resolve({ location: 'granted' });
    }
    start(config) {
        if (!BgLocWeb.timer) {
            BgLocWeb.timer = setInterval(() => {
                navigator.geolocation.getCurrentPosition(position => {
                    let location = {
                        id: BgLocWeb.lastId++,
                        sessionToken: config.sessionToken,
                        time: position.timestamp,
                        latitude: position.coords.latitude,
                        longitude: position.coords.longitude,
                        altitude: position.coords.altitude
                            ? position.coords.altitude
                            : undefined,
                        speed: position.coords.speed ? position.coords.speed : undefined,
                        bearing: position.coords.heading
                            ? position.coords.heading
                            : undefined,
                        accuracy: position.coords.accuracy
                            ? position.coords.accuracy
                            : 10,
                        provider: 'HTML5',
                        nanos: position.timestamp * 1000,
                    };
                    BgLocWeb.records.push(location);
                    BgLocWeb.location.next(location);
                }, error => {
                    console.error('ERROR', error);
                }, {
                    enableHighAccuracy: config.priority === 100,
                });
            }, 2000);
            return Promise.resolve();
        }
        else {
            return Promise.reject('BgLoc (web) already running.');
        }
    }
    stop() {
        if (!!BgLocWeb.timer) {
            clearInterval(BgLocWeb.timer);
            BgLocWeb.timer = undefined;
            return Promise.resolve();
        }
        else {
            return Promise.reject();
        }
    }
    getLocations(options) {
        let arr = BgLocWeb.records
            .filter(x => x.sessionToken === options.sessionToken)
            .sort((a, b) => a.id - b.id)
            .slice(0, options.limit
            ? Math.min(options.limit, BgLocWeb.records.length) - 1
            : BgLocWeb.records.length - 1);
        return Promise.resolve({
            items: arr,
        });
    }
    getStoredLocationsCount() {
        return Promise.resolve({
            count: BgLocWeb.records.length,
        });
    }
    getAll() {
        return Promise.resolve({
            items: BgLocWeb.records,
        });
    }
    confirmReception(options) {
        for (let id of options.ids) {
            let idx = BgLocWeb.records.findIndex(x => x.sessionToken === options.sessionToken && x.id === id);
            if (idx > -1) {
                BgLocWeb.records.splice(idx, 1);
            }
        }
        return Promise.resolve();
    }
    upload(config) {
        console.log('BgLoc (web) upload triggered (not supported)', config);
        return Promise.resolve();
    }
    cleanup(options) {
        if (!options) {
            BgLocWeb.records = [];
        }
        else {
            let ids = [];
            if (options.ids) {
                ids = options.ids;
            }
            else if (options.sessionToken) {
                ids = BgLocWeb.records
                    .filter(x => x.sessionToken === options.sessionToken)
                    .map(x => x.id);
            }
            while (ids.length > 0) {
                let idx = BgLocWeb.records.findIndex(x => x.id === ids[0]);
                BgLocWeb.records.splice(idx, 1);
                ids.splice(0, 1);
            }
        }
        return Promise.resolve();
    }
    addListener(eventName, listenerFunc) {
        let handle;
        switch (eventName) {
            case 'onLocationChanged':
                let subscription = BgLocWeb.location.subscribe((x) => {
                    listenerFunc(x); // do use first arg "err" as response
                });
                handle = {
                    remove: async () => {
                        await subscription.unsubscribe();
                    },
                };
                break;
            case 'onIdleTimeExceeded':
                console.warn(`Timeout event not supported by web platform.`);
                handle = {
                    remove: async () => {
                        await subscription.unsubscribe();
                    },
                };
                break;
            default:
                console.warn(`Invalid BgLoc event ${eventName}.`);
                throw `Invalid BgLoc event ${eventName}.`;
        }
        const result = Promise.resolve(handle);
        result.remove = handle;
        return result;
    }
    on(eventName) {
        switch (eventName) {
            case InfaBgLocEvent.LOCATION:
                return BgLocWeb.location;
            case InfaBgLocEvent.TIMEOUT:
                console.warn(`Timeout event not supported by web platform.`);
                return new Observable();
            default:
                console.warn(`Invalid BgLoc event ${eventName}.`);
                return new Observable();
        }
    }
    async echo(options) {
        console.log('ECHO', options);
        return options;
    }
}
BgLocWeb.timer = undefined;
BgLocWeb.lastId = 1;
BgLocWeb.records = [];
BgLocWeb.location = new Subject();
//# sourceMappingURL=web.js.map